#!/bin/sh

#################
# SHELL OPTIONS #
#################

# make wildcard globbing with no match return null instead of literal
shopt -s nullglob
# make wildcard globbing case-insensitive (such as WAV vs. wav)
shopt -s nocaseglob

########################
# FUNCTION DEFINITIONS #
########################

# function to wait for user interaction
pause()
{
	echo "Press any key to continue . . ."
	while [ true ] ; do
		# poll keyboard every second to read one character
		# suppresses echo of character to console
		read -s -t 1 -n 1 <&1
		if [ $? = 0 ] ; then
			# quit loop when user actually makes an entry
			break;
		fi
	done
}

# function to run any time script needs to abort
enditall()
{
	pause
	# close terminal without user intervention
	osascript -e 'tell application "Terminal" to close (every window whose name contains "EFE2IMG")' & exit 1
}

###############
# MAIN SCRIPT #
###############

# change terminal window name
echo -n -e "\033]0;EFE2IMG\007"

# change terminal window dimensions
echo -n -e "\033[8;54;80t"

# get directory that launched script and use it as current directory
DIR=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )
cd "${DIR}"

# create the [IMG_CHK_IN] directory (with error suppression if already exists)
mkdir ./\[IMG_CHK_IN\] 2>/dev/null

clear
echo "##############################################"
echo "# Ensoniq IMG (ASR/EPS) Image Verifier v0.70 #"
echo "##############################################"
echo ""
echo "Input files must be Ensoniq raw images placed within the [IMG_CHK_IN] folder. "
echo "All images must have an *.img file extension, as generated by various Ensoniq "
echo "utilities such as EpsLin, EPSDisk, EnsoniqFS, and Translator. "
echo ""
echo "Sub-directories are supported. "
echo ""
echo "No changes will be made to any of the files. This utility is read only. "
echo ""
pause
echo ""

# check for required tools and metadata
if [ ! -f "./tools/epslin" ]; then
	echo "Critical Error: epslin cannot be found. Aborting! \n"
	enditall
fi

# check for any present IMG files
# note: done recursively so sub-directories are supported
foundIMGs=$(find ./\[IMG_CHK_IN\] -type f \( -iname '*.img' \) | wc -l)
if [ $foundIMGs -eq 0 ]; then
	echo "Aborting: No IMGs found. \n"
	enditall
fi

find ./\[IMG_CHK_IN\] -type f \( -iname '*.img' \) -print0 |
while IFS= read -r -d '' imgfile; do
	# strip path to derive filename only (without path or extension)
	imgname=$(basename -- "${imgfile}")
	imgname=${imgname%.*}
	echo "Checking ${imgname}... \n"
  ./tools/epslin -D "${imgfile}"
  ./tools/epslin -C0 "${imgfile}"
	pause
done

# close terminal without user intervention
osascript -e 'tell application "Terminal" to close (every window whose name contains "EFE2IMG")' & exit 0
